<?php

/**
 * Settings form for the current module.
 *
 * @param type $form
 * @param type $form_state
 * @return type
 */
function drd_server_settings(&$form_state) {
  $form = array();
  $form['drd_allowed_referer'] = array(
    '#type' => 'textfield',
    '#title' => t('Allowed referer IP addresses'),
    '#default_value' => variable_get('drd_allowed_referer', ''),
    '#description' => t('Separate multiple values with a space character.')
  );
  $form['info'] = array(
    '#type' => 'markup',
    '#value' => '<pre>' . print_r(variable_get('drd_aes_keys', array()), TRUE) . '</pre>',
    '#title' => t('Currently available AES keys'),
  );
  $form['#submit'][] = 'drd_server_settings_submit';
  return system_settings_form($form);
}

/**
 * Submit handler for the settings form.
 *
 * @param type $form
 * @param type $form_state
 */
function drd_server_settings_submit($form, $form_state) {
  $ips = explode(' ', $form_state['values']['drd_allowed_referer']);
  $aes_keys = variable_get('drd_aes_keys', array());
  foreach ($aes_keys as $key => $def) {
    $aes_keys[$key]['keep'] = FALSE;
  }
  $changed = FALSE;
  if (!empty($aes_keys)) {
    $keys = array_keys($aes_keys);
    $master = $aes_keys[$keys[0]];
    foreach ($ips as $ip) {
      if (!array_key_exists($ip, $aes_keys)) {
        $aes_keys[$ip] = $master;
        $aes_keys[$ip]['keep'] = TRUE;
        $changed = TRUE;
      }
      else {
        $aes_keys[$ip]['keep'] = TRUE;
      }
    }
  }
  foreach ($aes_keys as $key => $def) {
    if (!$aes_keys[$key]['keep']) {
      unset($aes_keys[$key]);
      $changed = TRUE;
    }
  }
  if ($changed) {
    variable_set('drd_aes_keys', $aes_keys);
    $sites = drd_server_read_sites();
    drd_server_key_remote($sites, $aes_keys);
  }
}
