<?php

/**
 * DRD Server Operation to retrieve all kind of information from the current domain.
 *
 * @return type
 */
function drd_server_domain_info() {
  if (module_exists('mollom')) {
    $mollom_status = _mollom_status(TRUE);
    if (is_array($mollom_status)) {
      module_disable(array('mollom'));
    }
  }
  $result = array(
    'drd_db' => array(
      'drd_domain' => array(
        'mode' => 'update',
        'field' => 'name',
        'value' => variable_get('site_name', ''),
        'where' => 'did = @id',
      ),
      'drd_server' => array(
        'mode' => 'update',
        'field' => 'drupalversion',
        'value' => VERSION,
        'where' => 'sid = @sid',
      ),
    ),
  );

  // Load .install files
  include_once DRUPAL_ROOT . '/includes/install.inc';
  drupal_load_updates();
  // Check run-time requirements and status information.
  $requirements = module_invoke_all('requirements', 'runtime');
  usort($requirements, '_system_sort_requirements');

  $result['system'] = array(
    'status_warning' => (drupal_requirements_severity($requirements) == REQUIREMENT_WARNING),
    'status_error' => (drupal_requirements_severity($requirements) == REQUIREMENT_ERROR),
  );

  $server_values = array(
    t('Drupal'),
    t('Coder Upgrade directory'),
    t('Drupal core update status'),
    t('GD library'),
    t('PHP'),
    t('PHP memory limit'),
    t('PHP register globals'),
    t('Unicode library'),
    t('Upload progress'),
    t('Web server'),
  );

  $info = array(
    'title' => 'Information',
    'callback' => 'theme',
    'arguments' => array('status_report', 'variables' => array('requirements' => array())),
  );
  $result['cache']['info']['server'] = $info;
  $result['cache']['info']['domain'] = $info;
  foreach ($requirements as $key => $values) {
    $area = in_array($values['title'], $server_values) ? 'server' : 'domain';
    $result['cache']['info']['domain']['arguments']['variables']['requirements'][] = $requirements[$key];
    if ($area == 'server') {
      $result['cache']['info']['server']['arguments']['variables']['requirements'][] = $requirements[$key];
    }
  }
  //include more details in the result array for domains
  // e.g. info from the settings.php
  return drd_server_result('info', $result);
}

/**
 * DRD Server Operation to flush all caches from the current domain.
 *
 * @return type
 */
function drd_server_domain_flush_cache($redirect = FALSE) {
  drupal_flush_all_caches();
  drupal_set_message('Caches cleared');
  if ($redirect) {
    drupal_goto($_SERVER['HTTP_REFERER']);
  }
  return drd_server_result('cache.flush', '');
}

/**
 * DRD Server Operation to run cron on the current domain.
 *
 * @return type
 */
function drd_server_domain_run_cron() {
  drupal_cron_run();
  drupal_set_message('Cron completed');
  return drd_server_result('run.cron', '');
}

/**
 * DRD Server Operation to determine all available updates for the current domain.
 *
 * @return type
 */
function drd_server_domain_list_updates($mode = 'domain') {
  if (!module_exists('update')) {
    drupal_set_message(t('Update module not installed.'));
    return drd_server_result('list.updates', '');
  }

  update_refresh();
  module_load_include('inc', 'update', 'update.report');
  $result['cache']['update'][$mode] = array(
    'title' => 'Available Updates',
    'callback' => 'drupal_render',
    'arguments' => array(
      'data' => array(
        '#markup' => update_status(),
      ),
    ),
  );
  return drd_server_result('list.updates', $result);
}

/**
 * DRD Server Operation to run update.php on the current domain.
 *
 * @return type
 */
function drd_server_domain_run_update() {
  global $user;
  $user = user_load(1);
  variable_set('site_offline', TRUE);
  $batch = array(
    'operations' => array(array('drd_server_domain_find_required_updates', array())),
  );
  batch_set($batch);
  $batch =& batch_get();
  $batch['progressive'] = FALSE;
  $_GET['token'] = drupal_get_token('update');
  $_REQUEST['op'] = 'Update';
  $_POST['start'] = array();
  include_once './update.php';
  variable_set('site_offline', FALSE);
  drupal_set_message('Update completed');
  return drd_server_result('run.update', '');
}

/**
 * DRD Server Operation to update all translations on the current domain.
 *
 * @return type
 */
function drd_server_domain_update_translation() {
  drupal_set_message('Update translation is not available on Drupal 6');
  return drd_server_result('update.translation', '');
}

/**
 * DRD Server Operation to switch maintenance mode on/off on the current domain.
 *
 * @return type
 */
function drd_server_domain_switch_maintenance($flag) {
  variable_set('site_offline', $flag);
  $text = $flag ? 'Turned maintenance mode on' : 'Turned maintenance mode off';
  drupal_set_message($text);
  return drd_server_result('switch.maintenance', '');
}

/**
 * Internal function called by update.php
 */
function drd_server_domain_find_required_updates() {
  drupal_get_messages();
  $form = update_script_selection_form();
  $operations = array();
  foreach ($form['start'] as $module => $def) {
    if (isset($def['#default_value'])) {
      $version = $def['#default_value'];
      drupal_set_installed_schema_version($module, $version - 1);
      $updates = drupal_get_schema_versions($module);
      if ($updates) {
        $max_version = max($updates);
        if ($version <= $max_version) {
          foreach ($updates as $update) {
            if ($update >= $version) {
              $operations[] = array('update_do_one', array($module, $update));
              watchdog('DRD', 'Updating '. $module .': version '. $update);
              drupal_set_message('Updating '. $module .': version '. $update);
            }
          }
        }
      }
    }
  }
  $operations[] = array('drd_server_domain_capture_update_messages', array());
  $batch = array(
    'operations' => $operations,
    'title' => 'Updating',
    'init_message' => 'Starting updates',
    'error_message' => 'An unrecoverable error has occurred. You can find the error message below. It is advised to copy it to the clipboard for reference.',
  );
  batch_set($batch);
}

/**
 * Internal function called by update.php
 */
function drd_server_domain_capture_update_messages() {
  $batch_set = &_batch_current_set();
  update_finished(TRUE, $batch_set['results'], array(), format_interval($batch_set['elapsed'] / 1000));
  drupal_set_message(update_results_page());
}

/**
 * Define name and password for the special user 1
 *
 * @param type $username
 * @param type $password1
 * @param type $password2
 * @return type
 */
function drd_server_domain_reset_user1($username, $password1, $password2) {
  $check = user_validate_name($username);
  if (!empty($check)) {
    return drd_server_error($check);
  }
  $user = user_load(array('name' => $username));
  if (!empty($user) && $user->uid !== 1) {
    return drd_server_error(t('Username already taken.'));
  }
  if (empty($password1)) {
    return drd_server_error(t('Password can not be empty.'));
  }
  if ($password1 !== $password2) {
    return drd_server_error(t('Passwords do not patch.'));
  }

  $account = user_load(1);
  $edit = array(
    'name' => $username,
    'pass' => $password1,
  );
  if (!user_save($account, $edit)) {
    return drd_server_error(t('Changing user credentials failed.'));
  }
  return drd_server_result('set.user1', '');
}
